#include "MyService.hh"
#include <rtm/CorbaNaming.h>
#include <rtm/NVUtil.h>
#include <string>
#include <iostream>

// 以前のMLに投稿されていた”サービスポートのRTC以外からの利用”のソースコードを参考にテストプログラムを作成してみました。

int main(int argc,char** argv){
    std::string str;
    std::string name;
    if (argc > 1 && strcmp(argv[1], "A") == 0) {
      str = "port.MyService.myservicea0";
      name = "MyServiceProviderA0";
    }
    else if (argc > 1 && strcmp(argv[1], "B") == 0) {
      str = "port.MyService.myserviceb0";
      name = "MyServiceProviderB0";
    }
    else {
      std::cout << "A? B?" << std::endl;
      return 1;
    }

//	// ネームサービスへのアクセス準備
	// ORBの初期化
	CORBA::ORB_var p_orb = CORBA::ORB_init(argc,argv);
	// ネームサービスアクセス用クラスCorbaNamingの生成&設定
	RTC::CorbaNaming naming(p_orb,"localhost:9876");

//	// RTコンポーネントのオブジェクト参照の取得
	// ネームサービスよりオブジェクト参照を取得
	CORBA::Object_var p_obj = naming.resolve(name.c_str());
	// オブジェクト参照をRTObjectクラスにナローイング
	RTC::RTObject_var p_comp = RTC::RTObject::_narrow(p_obj);
	// エラーチェック
	if((!p_comp) || is_nil(p_comp)){
		std::cout << " err : can't get Object ref. " << std::endl;
		return 0;
	}
	std::cout << " ok  : get Object ref. " << std::endl;

//	// ポートのオブジェクト参照を取得
//	// コンポーネントプロファイルの取得
	RTC::ComponentProfile* p_comp_prof = p_comp->get_component_profile();
	// エラーチェック
	if(!p_comp_prof){
		std::cout << " err : can't get ComponentProfile " << std::endl;
		return 0;
	}
//	// ポートプロファイルの取得
	RTC::PortProfileList port_profs = p_comp_prof->port_profiles;
	// エラーチェック
	CORBA::ULong port_profs_length = port_profs.length();
	if(port_profs_length < 1){
		std::cout << " err : can't get PortProfiles " << std::endl;
		return 0;
	}
	// ポートインターフェースの検索
	CORBA::ULong port_it = 99;
	CORBA::ULong if_it = 99;
	for(CORBA::ULong i = 0; i < port_profs_length; i++){
		std::cout << " port name [" << i << "] : " << port_profs[i].name << std::endl;
		// ポートインターフェースプロファイルを取得
		RTC::PortInterfaceProfileList port_if_profs = port_profs[i].interfaces;

		CORBA::ULong port_if_profs_length = port_if_profs.length();
		for(CORBA::ULong j = 0; j < port_if_profs_length; j++){
			std::cout << "   port IF name [" << j << "] : " << port_if_profs[j].instance_name << std::endl;
			std::cout << "   port IF type [" << j << "] : " << port_if_profs[j].type_name << std::endl;
			std::cout << "   port IF pol. [" << j << "] : " << (port_if_profs[j].polarity == RTC::PROVIDED ? "provided" : "Required") << std::endl;
			if(!strcmp(port_if_profs[j].type_name,"MyService")){
				std::cout << "   Bingo ! " << std::endl;
				port_it = i;
				if_it = j;
				break;
			}
		}
	}
	// 見付からず検索失敗の場合
	if((port_it == 99) || (if_it == 99)){
		std::cout << " err : no matching IF " << std::endl;
		return 0;
	}
	// ポートのオブジェクト参照を取得
	RTC::Port_var p_port = port_profs[port_it].port_ref;
	// エラーチェック
	if((!p_port) || is_nil(p_port)){
		std::cout << " err : can't get Port ref. " << std::endl;
		return 0;
	}

	std::cout << " ok  : get Port ref. " << std::endl;

//	// コネクタプロファイルを作成	
	RTC::ConnectorProfile cnct_prof;
	cnct_prof.name = CORBA::string_dup("MyServiceCalling");
	cnct_prof.connector_id = "";
	cnct_prof.ports.length(1);
	cnct_prof.ports[0] = p_port;
	cnct_prof.properties.length(0);
	// コネクト関数呼出()
	p_port->connect(cnct_prof);

	std::cout << " ok  : connected " << std::endl;

//	// サービスのオブジェクト参照を取得
	// コネクトした後のコネクタプロファイルに含まれるプロパティから、サービスのオブジェクト参照を取得
	NVUtil::find(cnct_prof.properties,str.c_str()) >>= CORBA::Any::to_object(p_obj);
	// エラーチェック
	if((!p_obj) || is_nil(p_obj)){
		std::cout << " err : can't get Port ref. " << std::endl;
		return 0;
	}

	// サービスの型にナローイング
	MyService_var mysvc = MyService::_narrow(p_obj);

//	// サービスの呼出
	std::string echo_string(" function invokation succes ! ");
	echo_string += name;
	mysvc->echo(echo_string.c_str());
	std::cout << " ok  : invoke func. " << std::endl;
}
